#!/usr/bin/bash
#author: yoyang@redhat.com

export LANG=C
infoecho() { echo -e "\n${P:-DO ==>}" "\E[1;34m" "$@" "\E[0m"; }
errecho() { echo -e "\n${P:-ERR ==>}" "\E[31m" "$@" "\E[0m"; }

IMG_NAME=${IMG_NAME:-"iscsi.img"}
IMG_SIZE=${IMG_SIZE:-"8192"}
BLOCK_NAME=${BLOCK_NAME:-"block11"}
IQN_NAME=${IQN_NAME:-"iqn.2015-07.com.example.domain11"}
TARGET_NAME=${TARGET_NAME:-"system11"}
ACL_NAME="${IQN_NAME}:${TARGET_NAME}"
pkgs="targetcli"

hostname `hostname -A 2>/dev/null | awk '{print $1}'`
host=$(hostname)
[ -z "$host" ] && {
	errecho "{WARN} There is no available host:name find. use local name instead."
	host=$(hostname -f)
	ping -c 3 "$host" || {
		errecho "{WARN} Can not ping hostname:'${host}', please ensure '$host' available."
		exit 1
	}
}
#===============================================================================
infoecho "make sure packages $pkgs installed ..."
rpm -q $pkgs || { yum -y install $pkgs >/dev/null 2>&1; rpm -q $pkgs; }

infoecho "got the local IPv4 and IPv6 address ..."
read IP4 _nil <<<$(getDefaultIp)
read IP6 _nil <<<$(getDefaultIp 6)
[ -z "$IP4" -o -z "$IP6" ] && {
	errecho "{WARN} Cannot get the IPv4 or IPv6 address"
	exit 1
}

infoecho "clean original configuration ..."
losetup | grep "$IMG_NAME" >/dev/null
[ $? = 0 ] && losetup -d `losetup | grep "$IMG_NAME" | awk '{print $1}'`
systemctl restart target
losetup | grep "$IMG_NAME" >/dev/null
[ $? != 1 ] && {
	errecho "{WARN} Cannot delete the original loop device"
	exit 1
}

infoecho "create iSCSI image file ..."
dd if=/dev/zero of=$IMG_NAME bs=1M count=$IMG_SIZE 2>/dev/null
[ $? != 0 ] && {
	errecho "{WARN} Cannot create the image file $IMG_NAME"
	exit 1
}

infoecho "set up on the first free loop device ..."
LOOP_DEV=$(losetup -f)
losetup $LOOP_DEV $IMG_NAME
[ $? != 0 ] && {
	errecho "{WARN} Cannot setup loop device"
	exit 1
}

infoecho "setup iSCSI target using targetcli ..."
targetcli /backstores/block/ create $BLOCK_NAME $LOOP_DEV
targetcli /iscsi/ create $ACL_NAME
targetcli /iscsi/$ACL_NAME/tpg1/acls/ create $ACL_NAME
targetcli /iscsi/$ACL_NAME/tpg1/luns/ create /backstores/block/$BLOCK_NAME
# Delete the default created portal listening on all IPs (0.0.0.0), port 3260
targetcli /iscsi/$ACL_NAME/tpg1/portals/ delete 0.0.0.0 3260
# Others will use hostname to discover the target, so include both IPv4 & IPv6
targetcli /iscsi/$ACL_NAME/tpg1/portals/ create $IP4
targetcli /iscsi/$ACL_NAME/tpg1/portals/ create $IP6

infoecho "check the iSCSI configuration ..."
targetcli ls

infoecho "check the target configuration ..."
for res in /sys/kernel/config/target/core/*/*/pr/res_*; do
	echo $res; cat $res;
done
