#!/usr/bin/bash
#author: yoyang@redhat.com

export LANG=C
infoecho() { echo -e "\n${P:-DO ==>}" "\E[1;34m" "$@" "\E[0m"; }
errecho() { echo -e "\n${P:-DO ==>}" "\E[31m" "$@" "\E[0m"; }

iSCSI_SERV=$1
EXTRA_ARG=$2
EXP_PATH=$3
IQN_NAME=${IQN_NAME:-"iqn.2015-07.com.example.domain11"}
TARGET_NAME=${TARGET_NAME:-"system11"}
ACL_NAME="${IQN_NAME}:${TARGET_NAME}"
iSCSI_INIT_CONF="/etc/iscsi/initiatorname.iscsi"
iSCSI_DEV=""
pkgs="iscsi-initiator-utils xfsprogs"

[ $# = 0 ] && {
	echo "usage: $0  <iscsi_server_name>  [xfs path|cleanup]"
	echo ""
	echo "EX: $0"
	echo "	Only setup iSCSI session (apply for pNFS client)"
	echo ""
	echo "EX: $0 xfs path"
	echo "	Setup iSCSI session, format the block device with xfs and"
	echo "	then mount it on the specified path (apply for pNFS server)"
	echo ""
	echo "EX: $0 cleanup"
	echo "	Logout the iSCSI session with iSCSI server"
	exit 1
}
[ -z "$iSCSI_SERV" ] && {
	errecho "{WARN} Something is wrong! Need to specify the iSCSI server"
	exit 1
}
ping -c 4 $iSCSI_SERV || {
	errecho "{WARN} Cannot cennect iSCSI server '$iSCSI_SERV'"
	exit 1
}
[ -n "$EXTRA_ARG" -a "$EXTRA_ARG" != "xfs" -a "$EXTRA_ARG" != "cleanup" ] && {
	errecho "{WARN} Cannot recognize the parameter, should be xfs or cleanup"
	exit 1
}
[ "$EXTRA_ARG" = "xfs" -a -z "$EXP_PATH" ] && {
	errecho "{WARN} Need to spedify the mount path if want to build xfs"
	exit 1
}
#===============================================================================
if [ "$EXTRA_ARG" = "cleanup" ]; then
	infoecho "logout the iscsi session with server ..."
	iscsiadm --mode node --targetname $ACL_NAME --portal $iSCSI_SERV --logout
	[ $? != 0 ] && {
		errecho "{WARN} Cannot logout the iscsi session"
		exit 1
	}
	exit 0
fi

infoecho "make sure packages $pkgs installed ..."
rpm -q $pkgs || { yum -y install $pkgs >/dev/null 2>&1; rpm -q $pkgs; }

infoecho "configure '$iSCSI_INIT_CONF' read by iscsid and iscsiadm ..."
echo "InitiatorName=$ACL_NAME" > $iSCSI_INIT_CONF

infoecho "start iscsid.service ..."
service iscsid restart
[ $? != 0 ] && {
	errecho "{WARN} Cannot start iscsid"
	exit 1
}

infoecho "discover and login iSCSI target ..."
iscsiadm --mode discoverydb --type sendtargets --portal $iSCSI_SERV --discover
iscsiadm --mode node --targetname $ACL_NAME --portal $iSCSI_SERV --login
[ $? != 0 ] && {
	errecho "{WARN} Cannot login the iscsi session"
	exit 1
}


if [ "$EXTRA_ARG" = "xfs" ]; then
	infoecho "wait and find the new device ..."
	time=0
	find /dev/disk -name "*$TARGET_NAME*" | grep "$TARGET_NAME" >/dev/null
	while [ $? != 0 ]; do
		# At most wait for 60 seconds to wait /dev/disk updating
		if [ $time -lt 30 ]; then
			sleep 2
			time=$(($time+1))
			find /dev/disk -name "*$TARGET_NAME*" | grep "$TARGET_NAME" >/dev/null
		else
			break
		fi
	done

	infoecho "get the iSCSI device name ..."
	iSCSI_DEV=$(find /dev/disk -name "*$TARGET_NAME*" | xargs readlink -e)
	if [ -z $iSCSI_DEV ]; then
		errecho "{WARN} Cannot find the iscsi device name"
		iscsiadm --mode node --targetname $ACL_NAME --portal $iSCSI_SERV --logout
		exit 1
	fi

	infoecho "construct an XFS filesystem to the device ..."
	mkfs.xfs -f $iSCSI_DEV
	[ $? != 0 ] && {
		errecho "{WARN} Cannot construct the XFS fs"
		exit 1
	}

	infoecho "mount the XFS device on $EXP_PATH ..."
	mount $iSCSI_DEV $EXP_PATH
	[ $? != 0 ] && {
		errecho "{WARN} Cannot mount the XFS device"
		exit 1
	}
	exit 0
fi
